if not StellarUF then return end

StellarCast = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0")

local f, overlay, spark, text
local current, total, chan, already, rat


local function ResetAll()
	chan = nil
	current = nil
	spark:SetPoint("CENTER", overlay, "LEFT", 0, 0)
	f:SetBackdropColor(0, 0, 0, 0.2)
	overlay:Hide()
end
local function OnUpdate()
	if not current or current >= length then
		ResetAll()
	else
		current = current + arg1
		if chan then
			spark:SetPoint("CENTER", overlay, "LEFT", (length - current) * rat, 0)
		else
			spark:SetPoint("CENTER", overlay, "LEFT", current * rat, 0)
		end
	end
end
local function BuildTick()
	f = StellarUF.units.target
	overlay = CreateFrame("Frame", nil, f)
	overlay:ClearAllPoints()
	overlay:SetAllPoints(f)
	overlay:SetFrameStrata("HIGH")
	overlay:SetScript("OnUpdate", OnUpdate)
	overlay:Hide()

	spark = overlay:CreateTexture(nil, "OVERLAY")
	spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
	spark:SetWidth(10)
	spark:SetHeight(f:GetHeight() * 2.3)
	spark:SetBlendMode("ADD")
	spark:SetAlpha(0.8)
	spark:SetPoint("CENTER", overlay, "LEFT", 0, 0)
	
	text = overlay:CreateFontString(nil, "OVERLAY")
	text:ClearAllPoints()
	text:SetPoint("TOPLEFT", overlay, "TOPLEFT", 0, - f:GetHeight() / 4.2)
	text:SetPoint("BOTTOMRIGHT", overlay, "BOTTOMRIGHT", 0, 0)
	text:SetJustifyH("CENTER")
	text:SetJustifyV("CENTER")
	text:SetFont(GameFontHighlightSmall:GetFont(), f:GetHeight() / 3.5)
	text:SetShadowColor(0.1, 0.1, 0.1)
	text:SetShadowOffset(1,-1)
	text:SetTextColor(1, 0.2, 0.2)
	text:SetAlpha(0.7)
	text:SetText("")
	return true
end


function StellarCast:OnInitialize()
	if not BuildTick() then return end

	self:RegisterEvent("UNIT_SPELLCAST_START")
	self:RegisterEvent("UNIT_SPELLCAST_STOP", "CastStop")
	self:RegisterEvent("UNIT_SPELLCAST_FAILED", "CastStop")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED", "CastStop")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED", "CastStop")
	self:RegisterEvent("UNIT_SPELLCAST_DELAYED")
	
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
	
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
end

function StellarCast:PLAYER_TARGET_CHANGED()
	ResetAll()
end

function StellarCast:UNIT_SPELLCAST_START()
	if arg1 ~= "target" then return end
	
	local _,_,distext,_,startTime,endTime = UnitCastingInfo("target")
	if not distext then return end
	
	current = GetTime() - (startTime / 1000)
	length = (endTime - startTime) / 1000
	chan = nil
	rat = f:GetWidth() / length
	text:SetText(distext)
	overlay:Show()
	f:SetBackdropColor(0.7, 0, 0, 0.8)
end

function StellarCast:CastStop()
	if arg1 ~= "target" or chan then return end
	current = nil
end

function StellarCast:UNIT_SPELLCAST_DELAYED()
	if arg1 ~= "target" then return end
	local _,_,distext,_,startTime, endTime = UnitCastingInfo("target")
	if not distext then return end
	
	current = GetTime() - (startTime / 1000)
	length = (endTime - startTime) / 1000
	rat = f:GetWidth() / length
	text:SetText(distext)
end

function StellarCast:UNIT_SPELLCAST_CHANNEL_START()
	if arg1 ~= "target" then return end
	local spell, _, _, _, startTime, endTime = UnitChannelInfo("target")
	if not spell then return end
	
	current = GetTime() - (startTime / 1000)
	length = (endTime - startTime) / 1000
	chan = true
	rat = f:GetWidth() / length
	text:SetText(spell)
	overlay:Show()
	f:SetBackdropColor(0.7, 0.2, 0, 0.8)
end

function StellarCast:UNIT_SPELLCAST_CHANNEL_UPDATE()
	if arg1 ~= "target" then return end
	local spell, _, _, _, startTime, endTime = UnitChannelInfo("target")
	if not spell then return end
	
	current = GetTime() - (startTime / 1000)
	length = (endTime - startTime) / 1000
	rat = f:GetWidth() / length
	chan = true
end

function StellarCast:UNIT_SPELLCAST_CHANNEL_STOP()
	if arg1 ~= "target" then return end
	current = nil
	chan = nil
end

